@@ -108,30 +108,6 @@ temp_files = shared.configuration.get_temp_files()
 logger = logging.getLogger(__file__)
 
 def build(src, result_libs, args=[]):
-  # if a library is a .a, also build the .bc, as we need it when forcing a
-  # a system library - in that case, we always want all the code linked in
-  if result_libs:
-    need_forced = []
-    with_forced = []
-    for result_lib in result_libs:
-      if result_lib.endswith('.a'):
-        short = result_lib[:-2]
-        if short in SYSTEM_TASKS:
-          need_forced.append(short.replace('_noexcept', ''))
-          with_forced.append(short + '.bc')
-          continue
-      with_forced.append(result_lib)
-
-    if need_forced:
-      if os.environ.get('EMCC_FORCE_STDLIBS'):
-        print('skipping forced (.bc) versions of .a libraries, since EMCC_FORCE_STDLIBS already set')
-      else:
-        os.environ['EMCC_FORCE_STDLIBS'] = ','.join(need_forced)
-        try:
-          build(src, with_forced, args)
-        finally:
-          del os.environ['EMCC_FORCE_STDLIBS']
-
   # build in order to generate the libraries
   # do it all in a temp dir where everything will be cleaned up
   temp_dir = temp_files.get_dir()
