@@ -1,3 +1,8 @@
+// Copyright 2015 The Emscripten Authors.  All rights reserved.
+// Emscripten is available under two separate licenses, the MIT license and the
+// University of Illinois/NCSA Open Source License.  Both these licenses can be
+// found in the LICENSE file.
+
 // Pthread Web Worker startup routine:
 // This is the entry point file that is loaded first by each Web Worker
 // that executes pthreads on the Emscripten application.
@@ -56,8 +61,8 @@ function threadAlert() {
   var text = Array.prototype.slice.call(arguments).join(' ');
   postMessage({cmd: 'alert', text: text, threadId: selfThreadId});
 }
-Module['print'] = threadPrint;
-Module['printErr'] = threadPrintErr;
+out = threadPrint;
+err = threadPrintErr;
 this.alert = threadAlert;
 
 // #if WASM
@@ -126,7 +131,7 @@ this.onmessage = function(e) {
       STACK_MAX = STACK_BASE + e.data.stackSize;
       assert(STACK_BASE != 0);
       assert(STACK_MAX > STACK_BASE);
-      establishStackSpace(e.data.stackBase, e.data.stackBase + e.data.stackSize);
+      Module['establishStackSpace'](e.data.stackBase, e.data.stackBase + e.data.stackSize);
       var result = 0;
 //#if STACK_OVERFLOW_CHECK
       if (typeof writeStackCookie === 'function') writeStackCookie();
@@ -143,7 +148,7 @@ this.onmessage = function(e) {
         // enable that to work. If you find the following line to crash, either change the signature
         // to "proper" void *ThreadMain(void *arg) form, or try linking with the Emscripten linker
         // flag -s EMULATE_FUNCTION_POINTER_CASTS=1 to add in emulation for this x86 ABI extension.
-        result = Module['asm'].dynCall_ii(e.data.start_routine, e.data.arg);
+        result = Module['dynCall_ii'](e.data.start_routine, e.data.arg);
 
 //#if STACK_OVERFLOW_CHECK
         if (typeof checkStackCookie === 'function') checkStackCookie();
@@ -176,7 +181,7 @@ this.onmessage = function(e) {
         _emscripten_current_thread_process_queued_calls();
       }
     } else {
-      Module['printErr']('pthread-main.js received unknown command ' + e.data.cmd);
+      err('pthread-main.js received unknown command ' + e.data.cmd);
       console.error(e.data);
     }
   } catch(e) {
