@@ -1,6 +1,11 @@
 /*
-   malloc/free for SPLIT_MEMORY
-*/
+ * Copyright 2015 The Emscripten Authors.  All rights reserved.
+ * Emscripten is available under two separate licenses, the MIT license and the
+ * University of Illinois/NCSA Open Source License.  Both these licenses can be
+ * found in the LICENSE file.
+ *
+ * malloc/free for SPLIT_MEMORY
+ */
 
 #include <stdlib.h>
 #include <string.h>
