@@ -1,3 +1,8 @@
+# Copyright 2011 The Emscripten Authors.  All rights reserved.
+# Emscripten is available under two separate licenses, the MIT license and the
+# University of Illinois/NCSA Open Source License.  Both these licenses can be
+# found in the LICENSE file.
+
 '''
 Processes an LLVM assembly (.ll) file, adding debugging information.
 
@@ -255,7 +260,7 @@ for i in range(len(lines)):
           continue
 
   finally:
-    if len(pre) > 0:
+    if len(pre):
       lines[i] = pre + '\n' + lines[i]
       lines_added += 1
 
